@@ -6,7 +6,7 @@
  * Released to the public domain.
  *
  *--------------------------------------------------------------------------
- * $Id: hashtable.h,v 9999.27 2019/08/25 10:12:06 cvs Exp $
+ * $Id: hashtable.h,v 9999.30 2019/12/05 03:05:37 cvs Exp $
 \*--------------------------------------------------------------------------*/
 
 #ifndef _HASHTABLE_H
@@ -54,10 +54,12 @@ typedef struct {
 	void * appendix1;
 } ArrayList;
 
+int ArrayListContainsPtr(ArrayList * list, void * who);
 ArrayList * ArrayListCreate(int init_capacity);
 ArrayList * ArrayListDuplicate(ArrayList * ori);
 void ArrayListDestroy(ArrayList * list);
 void * ArrayListGet(ArrayList * list, srInt_64 n);
+void * ArrayListSet(ArrayList * list, srInt_64 n, void * new_elem);
 void * ArrayListRandom(ArrayList * list);
 int ArrayListPush(ArrayList * list, void * new_elem);
 int ArrayListPush_NoRepeatedPtr(ArrayList * list, void * new_elem);
@@ -71,10 +73,11 @@ int ArrayListPush_NoRepeatedPtr(ArrayList * list, void * new_elem);
 void * ArrayListShift(ArrayList * list);
 void * ArrayListPop(ArrayList * list);
 void ArrayListSetDeallocationFunction(ArrayList * list,  void (*elem_deallocator)(void *elem));
-void ArrayListSort(ArrayList * list, int compare_L_minus_R(void * L_elem, void * R_elem));
+void ArrayListSort(ArrayList * list, int compare_L_minus_R(void * L_elem, void * R_elem, ArrayList * me));
 
 // A simple comparison function if you want to sort unsigned long long ints.
 int ArrayListLLUComparison(void * L_elem, void * R_elem);
+int ArrayListStringComparison(void * L_elem, void * R_elem);
 
 // This function returns the index of the value in the list that is the minumum value greater than "value_less_than_dent".
 // The elements in the list are all zero-based unsigned long long ints. The list must be sorted (small -> large) before calling this function.
@@ -115,6 +118,7 @@ ArrayList * HashTableKeyArray(HashTable * tab);
 \*--------------------------------------------------------------------------*/
 
 HashTable * StringTableCreate(srInt_64 numOfBuckets);
+HashTable * StringTableReverse(HashTable * ori);
 HashTable *HashTableCreate(srInt_64 numOfBuckets);
 
 /*--------------------------------------------------------------------------*\
@@ -494,6 +498,11 @@ srUInt_64 HashTableStringHashFunction(const void *key);
 void free_values_destroy(HashTable * tab);
 int HashTablePutReplaceEx(HashTable *hashTable, const void *key, void *value, int replace_key, int dealloc_key, int dealloc_value);
 
+ArrayList * HashTableSortedIndexes(HashTable * k2int_tab, int larger_value_first);
+
+// NOTE: values are all added by 1. 
+HashTable * ArrayListToLookupTable_Int(ArrayList * arr);
+
 #endif /* _HASHTABLE_H */
 
 
