@@ -89,6 +89,7 @@ int iBLC_guess_scan(struct iBLC_scan_t * scancon, char * data_dir ){
 							scancon -> read_is_index[my_index-1]=is_idx;
 							scancon -> reads_per_cluster = max(scancon -> reads_per_cluster, my_index);
 						}else assert( my_index >0 && is_idx >=0 && rlen>0 );
+						//SUBREADprintf("LOAD CLUSTER %d = %d\n", my_index, rlen);
 					}
 				}
 				fclose(fp);
@@ -420,18 +421,27 @@ int cacheBCL_next_chunk(cache_BCL_t * cache_input){
 	return 0;
 }
 
+void iCache_copy_readbin(cache_BCL_t * cache_input, int * readlane, char * readbin, srInt_64 rno){
+	int bii, readno = cache_input -> read_no_in_chunk, blen=cache_input -> total_bases_in_each_cluster; 
+	for(bii = 0; bii < blen; bii++) readbin[bii] = cache_input -> bcl_bin_cache[bii][readno];
+
+	(* readlane) = cache_input -> lane_no_in_chunk[readno];
+	cache_input -> read_no_in_chunk=1+readno;
+	return;
+}
 
 int iCache_copy_read(cache_BCL_t * cache_input, char * read_name, char * seq, char * qual, srInt_64 rno){
 	int bii, idx_offset, base_offset;
 	int * srii = cache_input -> single_read_lengths;
 
 	#ifdef __MINGW32__
-	sprintf(read_name, "R%011I64u:", rno);
+	sprintf(read_name, "R%011" PRIu64 ":", rno);
 	#else
 	sprintf(read_name, "R%011llu:", rno);
 	#endif
+	int is_dual_index = srii[3]>0;
 	idx_offset  = srii[0];
-	base_offset = srii[1] + idx_offset;
+	base_offset = srii[1] + ( is_dual_index?srii[2]:0) + idx_offset;
 
 	read_name[13+idx_offset]='|';
 	read_name[14+2*idx_offset]='|';
@@ -450,25 +460,41 @@ int iCache_copy_read(cache_BCL_t * cache_input, char * read_name, char * seq, ch
 			nbase="ACGT"[nch%4];
 			nqual=33+(nch>>2);
 		}
-		if(nqual >= '/' && bii < srii[0] + srii[1]) nqual++;
+		if(nqual >= '/' && bii < base_offset ) nqual++;
 		if(bii < srii[0]){
 			read_name[13+bii] = nbase;
 			read_name[14+idx_offset+bii]= nqual;
-		}else if(bii < srii[0] + srii[1]){
+		}else if(bii < base_offset ){
 			read_name[15+idx_offset+bii] = nbase;
 			read_name[16+base_offset+bii]= nqual;
 		}else{
-			seq[bii - srii[0]-srii[1] ] = nbase;
-			qual[bii - srii[0]-srii[1] ] = nqual;
+			seq[bii - base_offset  ] = nbase;
+			qual[bii - base_offset  ] = nqual;
 		}
 	}
 
 	cache_input -> read_no_in_chunk++;
 	//SUBREADprintf("GOT READ #%d ; ret=%d\n" , cache_input -> read_no_in_chunk, srii[2]);
-	return srii[2];
+	return srii[2+is_dual_index];
 }
 
+int cacheBCL_next_readbin(cache_BCL_t * cache_input, int * readlane, char rbin[BCL_READBIN_ITEMS_LOCAL][BCL_READBIN_SIZE], int max_readbin_buffer, srInt_64 * start_allread_no){
+	int ii;
+	srInt_64 rnumb;
+	for(ii=0; ii< max_readbin_buffer; ii++){
+		if(cache_input -> read_no_in_chunk >= cache_input -> reads_available_in_chunk){
+			if(cache_input -> last_chunk_in_cache) break; 
+			cacheBCL_next_chunk(cache_input);
+			if(cache_input -> read_no_in_chunk >= cache_input -> reads_available_in_chunk) // no reads are loaded in the previous step
+				break;
+		}
 
+		rnumb =(cache_input -> chunk_no -1)*1ll * cache_input -> reads_per_chunk +(cache_input -> read_no_in_chunk);
+		if(!ii) (*start_allread_no) = rnumb;
+		iCache_copy_readbin(cache_input, readlane+ii, rbin[ii], rnumb);
+	}
+	return ii;
+}
 
 int cacheBCL_next_read(cache_BCL_t * cache_input, char * read_name, char * seq, char * qual, srInt_64 * read_number_in_all){
 	if(cache_input -> read_no_in_chunk >= cache_input -> reads_available_in_chunk){
@@ -500,7 +526,7 @@ int iBLC_current_lane_next_read(input_BLC_t * blc_input, char * readname , char
 	int bii, idx_offset, base_offset;
 
 	#ifdef __MINGW32__
-	sprintf(readname, "R%011I64u:", blc_input -> read_number +1);
+	sprintf(readname, "R%011" PRIu64 ":", blc_input -> read_number +1);
 	#else
 	sprintf(readname, "R%011llu:", blc_input -> read_number +1);
 	#endif
@@ -745,6 +771,7 @@ HashTable * input_BLC_parse_SampleSheet(char * fname){
 			stat1_line = 0;
 		}
 	}
+	fclose(fp);
 	if(state <1){
 		SUBREADprintf("ERROR: the sample sheet doesn't contain any sample.\n");
 		return NULL;
@@ -782,6 +809,57 @@ int is_ATGC(char c){
 	return c=='A'||c=='C'||c=='G'||c=='T'||c=='N';
 }
 
+int hamming_dist_ATGC_max3(char* s1, char* s2 ){
+	int xx,ret=0;
+	for(xx=0;;xx++){
+		char nch1 = s1[xx];
+		char nch2 = s2[xx];
+		if(is_ATGC(nch1) && is_ATGC(nch2)){
+			ret += nch1==nch2;
+			if(xx -ret >3) return 999;
+		}else break;
+	}
+	return xx-ret;
+}
+
+
+int hamming_dist_ATGC_max1_2p(char* s1, char* s2 ){
+	int xx,sl=0;
+	while(is_ATGC(s1[sl]) && is_ATGC(s2[sl])) sl++;
+	sl /=2;
+
+	int p1_mm=0, p2_mm=0;
+	for(xx=0;;xx++){
+		char nch1 = s1[xx];
+		char nch2 = s2[xx];
+		if(is_ATGC(nch1) && is_ATGC(nch2)){
+			if(nch1!=nch2){
+				if(xx<sl) p1_mm++;
+				else p2_mm++;
+			}
+		}else break;
+	}
+	if(p1_mm >1 || p2_mm>1 ) return 999;
+	return p1_mm+p2_mm;
+}
+
+
+
+int hamming_dist_ATGC_max1(char* s1, char* s2 ){
+	int xx,ret=0;
+	for(xx=0;;xx++){
+		char nch1 = s1[xx];
+		char nch2 = s2[xx];
+		if(is_ATGC(nch1) && is_ATGC(nch2)){
+			ret += nch1==nch2;
+			if(xx -ret >1) return 999;
+		}else break;
+	}
+	return xx-ret;
+}
+
+
+
 int hamming_dist_ATGC_max2(char* s1, char* s2 ){
 	int xx,ret=0;
 	for(xx=0;;xx++){
@@ -950,7 +1028,7 @@ int cacheBCL_qualTest_BAMmode(char * datadir, int testing_reads, int known_cell_
 		base[0]=qual[0]=rname[0]=0;
 		ret = scBAM_next_read(scBAM_input, rname , base, qual);
 		if(ret<=0)break;
-		char *cell_barcode = NULL, *sample_barcode=NULL, *lane_str=NULL;
+		char *cell_barcode = NULL;
 
 		int xx=0;
 		char *testi;
@@ -959,10 +1037,7 @@ int cacheBCL_qualTest_BAMmode(char * datadir, int testing_reads, int known_cell_
 				xx++;
 				if(xx == 1) {
 					cell_barcode = testi +1;
-				}else if(xx == 2) {
-					sample_barcode = testi +1;
 				}else if(xx == 4){
-					lane_str = testi+1;
 					break;
 				}
 			}
@@ -990,7 +1065,6 @@ int cacheBCL_qualTest_FQmode(char * datadir, int testing_reads, int known_cell_b
 		if(ret<=0)break;
 
 		char *cell_barcode = NULL;
-		//SUBREADprintf("RETV=%d, RN=%s\n" ,ret, rname);
 		int xx=0;
 		char *testi;
 		for(testi = rname+1; * testi; testi ++){
@@ -1004,7 +1078,8 @@ int cacheBCL_qualTest_FQmode(char * datadir, int testing_reads, int known_cell_b
 		}
 
 		int cell_no = iCache_get_cell_no(cell_barcode_table, cell_barcode_list, cell_barcode, known_cell_barcode_length);
-		if(cell_no>0) (*valid_cell_barcode)++;
+		//fprintf(stderr,"RETV=%d, RN=%s, CNO=%d, KCBL=%d\n" ,ret, rname, cell_no, known_cell_barcode_length);
+		if(cell_no>=0) (*valid_cell_barcode)++;
 
 		(*tested_reads)++;
 		if((*tested_reads) >= testing_reads)break;
@@ -1339,7 +1414,7 @@ int scBAM_next_read(input_scBAM_t * bam_input, char * read_name, char * seq, cha
 	char * extag_start = qual_start + l_seq;
 	char * tag_str_val=NULL, tag_type = 0;
 
-	for(x1=0;x1<5;x1++){
+	for(x1=0;x1<4;x1++){ // the "RG" isn't used for now
 		char * tagname = NULL;
 		if(x1==0)tagname ="CR";
 		if(x1==1)tagname ="UR";
@@ -1476,13 +1551,19 @@ int input_mFQ_next_read(input_mFQ_t * fqs_input, char * readname , char * read,
 		ret = autozip_gets(&fqs_input -> autofp1, tmpline, MAX_READ_NAME_LEN);
 		int write_ptr=0;
 		if(ret==0){
+			int R2ret = autozip_gets(&fqs_input -> autofp3, tmpline, MAX_READ_NAME_LEN);
+			if(R2ret >0){
+				SUBREADprintf("ERROR: the cell barcode and UMI reads exhausted before the genomic reads exhausted. The two FASTQ files seem to have different numbers of reads.\n");
+				return -2;
+			}
+
 			ret = input_mFQ_next_file(fqs_input);
 			if(ret >=0) continue;
-			return -1;
+			else return -1;
 		} else if(ret<0) return -1;
 
 		#ifdef __MINGW32__
-		sprintf(readname, "R%011I64d", fqs_input -> current_read_no);
+		sprintf(readname, "R%011" PRId64, fqs_input -> current_read_no);
 		#else
 		sprintf(readname, "R%011lld", fqs_input -> current_read_no);
 		#endif
@@ -1515,7 +1596,10 @@ int input_mFQ_next_read(input_mFQ_t * fqs_input, char * readname , char * read,
 		}else write_ptr+=sprintf(readname+write_ptr,"|input#%04d@L%03d", fqs_input -> current_file_no, fqs_input  -> current_guessed_lane_no);
 
 		ret = autozip_gets(&fqs_input -> autofp3, tmpline, MAX_READ_NAME_LEN);
-		if(ret<=0) return -1;
+		if(ret<=0){
+			SUBREADprintf("ERROR: the genomic reads exhausted before the cell barcode and UMI reads exhausted. The two FASTQ files seem to have different numbers of reads\n");
+			return -2;
+		}
 		ret = autozip_gets(&fqs_input -> autofp3, read, MAX_READ_LENGTH);
 		ret --; // read length excludes "\n"
 		read[ret]=0;
